(function($) {

    var SharePointListConfig = function() {
    };

    var sitesDropDown = null;

	SharePointListConfig.prototype.fields = {
        "string" : {
            "site" : function(param, options) {
                // create a basic field for the "site" parameter, based on the 'select' template.
                var paramDiv = $(Confluence.Templates.MacroBrowser.macroParameterSelect());
                sitesDropDown = $("select", paramDiv);

                // we need to do some trickery when the value is first set, and when the value is subsequently changed,
                // so set this up.
                if (options == null)
                    options = {};
                options.setValue = setSiteValue;

                // commence the asynchronous operation to retrieve the list of sites, pronto!
                AJS.SharePoint.bindSitesDropDown(sitesDropDown);

                return AJS.MacroBrowser.Field(paramDiv, sitesDropDown, options);
            },
            "list" : createSharePointAutoComplete("/rest/sharepoint/1.0/search",
                    function(val) {
                        return {
                            query: val.indexOf("/") > 0 ? val.substring(val.lastIndexOf("/") + 1) : val,
                            siteAlias: sitesDropDown.val()
                        };
                    },
                    function (e, selected) {
                        if (!selected) {
                            return;
                        }

                        // extract the information about the selected item from the jQuery data cache for the corresponding <span> element.
                        var dataObj = $.data(selected.find("span")[0], "properties");
                        $("#macro-param-view").val(dataObj.viewName);
                        $("#macro-param-list").val(dataObj.listName);

                        this.hide();
                        e.preventDefault();
                    }, true),
            "view" : createSharePointAutoComplete("/rest/sharepoint/1.0/search/view",
                        function (val) {
                            return {
                                query: val,
                                list: $("#macro-param-list").val(),
                                siteAlias: sitesDropDown.val()
                            };
                        },
                        function(e, selected) {
                            if (!selected) {
                                return;
                            }

                            var dataObj = $.data(selected.find("span")[0], "properties");
                            $("#macro-param-view").val(dataObj.name);

                            this.hide();
                            e.preventDefault();
                        }, false),
            "folder" : createSharePointAutoComplete("/rest/sharepoint/1.0/search/folder",
                    function(val) {
                        return {
                            query: val.indexOf("/") > 0 ? val.substring(val.indexOf("/") + 1) : val,
                            list: $("#macro-param-list").val(),
                            siteAlias: sitesDropDown.val()
                        };
                    },
                    function (e, selected) {
                        if (!selected) {
                            return;
                        }

                        // extract the information about the selected item from the jQuery data cache for the corresponding <span> element.
                        var dataObj = $.data(selected.find("span")[0], "properties");
                        $("#macro-param-folder").val(dataObj.relativePath);

                        this.hide();
                        e.preventDefault();
                    }, false)
        }
    };

    function setSiteValue(value) {
        // before we set the value, let's ensure that the appropriate option actually exists.
        // if this set is invoked by the macro browser as it loads a pre-existing macro, *and* the
        // asynchronous call to retrieve the list of available sites hasn't returned yet, then the option won't
        // be there.  If this is the case, then we'll temporarily add it in.
        var targetOption = $("#macro-param-site > option[value='" + value + "']");
        if (targetOption.length == 0) {
            // it doesn't exist; we need to create it.
            var option = $("<option/>");
            option.attr("value", value);
            option.attr("title", value);
            option.text(value);
            sitesDropDown.append(option);
        }

        // now set it.
        sitesDropDown.val(value);
    }

    function createSharePointAutoComplete(searchURL, makeParams, selectionHandler, required) {
        return function(param, options) {
            var paramDiv = $(Confluence.Templates.MacroBrowser.macroParameter());
            var input = $("input", paramDiv);

            if (required)
            {
                input.keyup(AJS.MacroBrowser.processRequiredParameters);
            }

            paramDiv.append("<div id=\"dd-list\" class=\"aui-dd-parent autocomplete\"></div>");
            var ddDiv = $("#dd-list", paramDiv);

            var ddOptions = {
                makeParams: makeParams,
                dropdownPlacement: function(dropDown) {
                    ddDiv.append(dropDown);
                },
                ajsDropDownOptions: {
                    selectionHandler: selectionHandler
                }
            };
            input.quicksearch(searchURL, null, ddOptions);
            return AJS.MacroBrowser.Field(paramDiv, input, options);
        }
    }

    AJS.MacroBrowser.Macros["sp-list"] = new SharePointListConfig();
})(AJS.$);